/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include <windows.h>
#include <stdio.h>

#include "serial.h"
#include "display.h"

#include "image.h"
#include "main.h"

#include "decompress.h"
#include "window.h"

#include "camera.h"

#include "log.h"

#define SIZE_160_120    0
#define SIZE_320_240    1
#define SIZE_640_480    2

#define HEADER_SIZE 5
#define FOOTER_SIZE 5

HANDLE h_camera_thread = NULL;               // Handel for serial control thread
DWORD camera_thread_id;               // Serial control 
bool camera_running = false;          // True if serial thread is running


CRITICAL_SECTION camera_thread_lock;  // Lock for serial thread


camera_class  camera(0x08);


camera_class::camera_class( unsigned char id )
{
    device_id = id;

//    camera_start();

}


camera_class::~camera_class( void )
{




}


/* Send get image command */
int camera_class::camera_start_image(void)
{
    add_line("Camera (%d) - Get image", device_id);

    unsigned char data[] = {0x56, 0x00, 0x36, 0x01, 0x00};
    
    if(serial_transfer(device_id, data, 5, data, 5)) return(1);

    // Need to check return word here

    return(0);
             
}
/* End of camera_get_image */


/* Send stop image command */
int camera_class::camera_stop_image()
{
    add_line("Camera - Stop image");
   
    unsigned char data[] = {0x56, 0x00, 0x36, 0x01, 0x03};
    
    if (serial_transfer(device_id, data, 5, data, 5)) return(1);
    // Need to check return word here

    Sleep(50);


    return(0);
}
/* End of camera_stop_image */


/* Send read block command */
int camera_class::camera_get_block(unsigned int &addr, unsigned int len, 
                      unsigned char *buff)
{
    log("Camera get block");
    
    unsigned char interval = 0x0a;

    unsigned char cmd[] = {0x56, 0x00, 0x32, 0x0c, 0x00, 0x0a, 0x00, 0x00,  
                           addr >> 8, addr & 0xff, 0x00, 0x00, 
                           len >> 8,  len & 0xff, 0x00, interval};


    // send command and get header
    if (serial_transfer(device_id, cmd, sizeof(cmd), cmd, 5)) return(1);

    // Get data
    if (serial_transfer(device_id, NULL, 0, buff, len)) return(1);

    // Get footer 
    if (serial_transfer(device_id, NULL, 0, cmd, 5)) return(1);

    addr += len;                  

    return(0);
}
/* End of camear_get_block */


/* Send Get JPEG filesize command */
int camera_class::camera_get_filesize(unsigned int &size)
{
    add_line("Camera - Get size");
    unsigned char data[] = {0x56, 0x00, 0x34, 0x01, 0x00, 0, 0, 0, 0};
    
    if (serial_transfer(device_id, data, 5, data, 9))
        return(1);

    size = (data[7] << 8) + data[8];
    
    add_line("Size: %d", size);

    // Need to check return word here

    return(0);
}
/* End of camera_get_filesize*/



/* Send set size command */
int camera_class::camera_set_size( int image_size )
{
    add_line("Camera - Set size");

    // Size command
    unsigned char cmd_160_120[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x22};
    unsigned char cmd_320_240[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x11};
    unsigned char cmd_640_480[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x00};

    unsigned char data[5];

    int ret = 1;
    
    if (image_size == SIZE_160_120)
    {
        ret = serial_transfer(device_id, cmd_160_120, sizeof(cmd_160_120), data, 5);
    } else
    if (image_size == SIZE_320_240)
    {
        ret = serial_transfer(device_id, cmd_320_240, sizeof(cmd_320_240), data, 5);
    } else
    if (image_size == SIZE_640_480)
    {
        ret = serial_transfer(device_id, cmd_640_480, sizeof(cmd_640_480), data, 5);
    } 

    camera_reset();

    return(ret);

}
/* End of camera_set_size */


/* Send set baud command */
int camera_class::camera_set_baud( void )
{
    add_line("Camera - Set baud");

    unsigned char data[] = {0x56, 0x00, 0x24, 0x03, 0x01, 0x0D ,0xA6};
   
    if (serial_transfer(device_id, data, 7, data, 5)) 
        return(1);
    
    // Reconnect at the higher baud rate
    stop_serial();
    start_serial(CBR_115200);
    
    return(0);
}
/* End of camera_set_baud */


/*  Reset the camera */
void camera_class::camera_reset()
{
    add_line("Camera - Reset");
 
    unsigned char data[] = {0x56, 0x00, 0x26, 0x00};
 
    bool init_ok = false;

    while (!init_ok)
    {
        if (serial_transfer(device_id, data, 4, data, 4) == 0)
            init_ok = 1;
           
        Sleep(2000);                             
    }    
  
    flush_serial();

    stop_serial();
    start_serial(CBR_38400);

    camera_set_baud();

    add_line("Camera - Init OK");
}
/* End of camera_reset */


/* Setup the image buffer */
void camera_class::camera_setup_image(int image_size)
{
    if (image_size == SIZE_160_120)
    {
        image_x_size = 160;
        image_y_size = 120;
    } else
    if (image_size == SIZE_320_240)
    {
        image_x_size = 320;
        image_y_size = 240;
    } else
    if (image_size == SIZE_640_480)
    {
        image_x_size = 640;
        image_y_size = 480;
    } 

    image_start();
}
/* End of camera_setup_image */



/* Get an image from the camera */
int camera_class::camera_get_image( unsigned char *buff, unsigned int &image_size )
{
    unsigned int addr = 0;
    unsigned int data_size = 0;

    bool image_done = false;
    bool image_error = false;

    if (camera_start_image()) return(1);
        
    if (camera_get_filesize(data_size)) return(1);

    unsigned int block_size = 0xf8;
                              
    while(!image_done)
    {  
        unsigned int read_len = block_size;
            
        // The read size for the last block is smaller than the boock size
        if (read_len > data_size - addr)
            read_len = data_size - addr;
            
        /* Send read block command */            
        if (camera_get_block(addr, read_len, (unsigned char * ) buff + addr))
        {
            add_line("Camera - Unable to transfer camera block");                
            image_error = 1; 
            break; 
        }
           
        if (addr == data_size)
            image_done = 1;
            
        if (!camera_running)
        {
            image_error = 1; 
            break; 
        
        }
            
    }
        
    image_size = addr;
        
    camera_stop_image();

    return(image_error);
}
/* End of camera_get_image */


/* The camera thread */
DWORD WINAPI camera_class::camera_thread(LPVOID p)
{
    camera_class *c = (camera_class *)p;


    bool done = false;
    
    int image_format = SIZE_160_120;
      
    //start_serial(CBR_38400);

    //camera_set_baud(); // Set fast baud rate
    // camera_set_size(image_size);  // Resize image abd buffers
        
    c->camera_setup_image(image_format);
    
    unsigned int image_size;
    
    while (!done)
    {
        EnterCriticalSection(&c->camera_thread_lock);
        done = !c->camera_running;
        LeaveCriticalSection(&c->camera_thread_lock);

        if(! c->camera_get_image((unsigned char *)compressed_image, image_size))
        {
            frames++;
       
            add_line("Camera - Decompress");
        
            decompress_image(compressed_image, image_size, image_buffer, image_x_size, image_y_size);
            InvalidateRect (main_win, NULL, FALSE);
        }

    }
        
    return(0);
}
/* End of camera thread */

DWORD camera_class::ThreadProc(LPVOID pvoid) 
{

	return (DWORD)1;
}




/* Starts the camera system */
void camera_class::camera_start( void )
{
    if (camera_running) return;


    InitializeCriticalSection(&camera_thread_lock);

    camera_running = true;

//	return ::CreateThread ((NULL, 0, (unsigned long (__stdcall *)(void *))this->runProcess, 
//				(void *)this, 0, NULL);

//    h_camera_thread = CreateThread(0, 0, camera_thread, NULL, 0, &camera_thread_id);
      
    

	//     DWORD dwThreadParam = (DWORD)this;
//	     DWORD dwThreadID;


    h_camera_thread  = CreateThread (NULL, 0, 
                                     (unsigned long (__stdcall *)(void *))camera_thread, 
                                     (void *)this, 0, &camera_thread_id);
    
    
    
    
    
}
/* End of camera_start */


/* Stop the camera */
void camera_class::camera_stop ( void )
{
    if (h_camera_thread != NULL)
    {
        EnterCriticalSection(&camera_thread_lock);
        camera_running = false;                         // Tell thread to exit
        LeaveCriticalSection(&camera_thread_lock);

        WaitForSingleObject(h_camera_thread, INFINITE); // Wait for thread to terminate
    }
}
/* End of camera stop */    
    
    
    









