/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include <windows.h>
#include <stdio.h>

#include "main.h"
#include "window.h"
#include "display.h"
#include "input.h"
#include "config.h"
#include "timer.h"
#include "camera.h"

#include "image.h"
#include "log.h"
#include "serial.h"

HWND system_hwnd = NULL;                        // Keep track of application handle

unsigned int frames = 0;                        // Numbers of frames so far in this second
float fps = 0;                           // Frames per second



/* Runs when program startsis created */
void system_startup()
{
    load_config("config.txt");

//    joystick_start();
//    robot_start();

    image_start();

    start_serial(CBR_38400);

    camera.camera_start();
   // camera2.camera_start();

//    network_start();
 
    SetTimer(system_hwnd, ID_SECOND, 4000, NULL);
    SetTimer(system_hwnd, ID_UPDATE, 200, NULL);  // Update robot 4 times a second
}
/* End of system_startup */


/* Runs when window is closed */
void system_shutdown(void)
{
   // robot_reset();

//    image_stop();
    
   // network_stop();
    
//    WSACleanup();

    camera.camera_stop();
    

    log_stop();

    PostQuitMessage(0);   
}
/* End of system_shutdown */



/* Deal with the system timer */
void timer_message(HWND hwnd, WPARAM wParam)
{
    if (wParam == ID_SECOND)
    {
        fps = frames/4.0;
        frames = 0;
      
//        network_check();            // Make sure network is still conneted
                                          
        return;
    } 
    
    if (wParam == ID_UPDATE)
    {
//        request_sensor_data();
//        joystick_check();
//        check_input();
//        robot_update();
//        update_display(hwnd);
//        InvalidateRect(hwnd, NULL, 0);  
        return;
    }    
}   
/* End of timer message */


/* Ehe message handler */
LRESULT CALLBACK WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                 
    {
        case WM_CREATE:     system_hwnd = hwnd;;    break;
        case WM_TIMER:      timer_message(hwnd, wParam); break;
        case WM_DESTROY:    system_shutdown(); break;
        case WM_KEYDOWN:    key_down(wParam, lParam); break;
        case WM_KEYUP:      key_up(wParam); break;

        case WM_PAINT:      do_paint(hwnd);  break;


        case WM_KILLFOCUS: input_lose_focus(); break;


        case WM_MOUSEMOVE:  process_mouse(lParam); break;
   
        case WM_SIZE:
            display_buffer_resize(hwnd, lParam);
            return(0);
            
//        case WM_NETEVENT: network_message(wParam, lParam); break;            
            
        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return(0);
}
/* End of winproc */


/* Winmain */
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    // Make the main window
    if (setup_window(hinstance, nCmdShow)) return(1);

    system_startup();

    MSG messages;

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }

    return(messages.wParam);
}
/* End of winmain */



