/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include <windows.h>
#include <stdio.h>

#include "serial.h"
#include "display.h"

#include "image.h"
#include "main.h"

#include "decompress.h"

#include "input.h"

#include "timer.h"
#include "window.h"
#include "log.h"
   

HANDLE h_serial = NULL;			      // Handle for serial port

bool serial_ok = false;               // True if serial is working 

bool timeout_flag = false;            // Set once for a timeout condition

char serial_port[10];                 // Filename of serial port


bool enable_mux  = false;




/* preform a transfer with the controller */
int serial_transfer(unsigned char device_id,
                    unsigned char *send_buff, int send_bytes,  
                    unsigned char *recv_buff, int recv_bytes)
{
    unsigned char data[] = {device_id, send_bytes};
    


    if (enable_mux)
    {
        if (serial_send (data, 2) == -1) 
            return(1);
    }

    if (send_bytes > 0)
    {
        if (serial_send (send_buff, send_bytes) == -1) 
            return(1);
    }


    if (enable_mux)
    {
        data[0] = 0;//calc_checksum();

        if (serial_send (data, 1) == -1) 
            return(1);
            
    }

    int ret = serial_read (recv_buff, recv_bytes);
        
    if (ret != recv_bytes)
    {
        add_line("Transfer: Unable to receive. Code: %d", ret);

        return(1);
    }
    
    return(0);
}
/* End of serial_transfer */





void show_buffer(char *header, unsigned char *data, int len, bool display)
{

    char item[5];
     
    unsigned int max = len * 5 + 100;
                

    char *buff = (char *) malloc(max);
                                           
    snprintf(buff, max, "(%d) %s:", len, header );
    
    for (int i = 0; i < len; i++)
    {
        sprintf(item, "%02x ", data[i]);
        strcat(buff, item);
    }

    log("%s", buff);
  
    free(buff);

}    
    


/* Make sure there is no pending data for the next retry */
void flush_serial( void )
{
    DWORD read = 1;
                    
    // Wait for byte
    while (read)
    {
        char ch;
        ReadFile (h_serial, &ch, 1, &read, NULL);  
    }  
} 
/* End of flush serial */



int serial_send (unsigned char *buffer, unsigned int len) 
{
    DWORD data_sent;
  
    show_buffer("SEND: ", buffer, len, 1);
  
   	// Send data to controller
   	if (!WriteFile(h_serial, buffer, len, &data_sent, NULL))
  	{
        add_line("Unable to send data");
        serial_ok = false;
        return(-1);
    }
        
    // Check data sent length 
    if (data_sent != len)
    {
        add_line("Send data error");
        serial_ok = false;        
//        return(1);
    }
    
    return(data_sent);
}




int serial_read(unsigned char *buffer, unsigned int len)
{                                                              
    DWORD data_read;

    // Read telemetry data from the controller 
    if (!ReadFile(h_serial, buffer, len, &data_read, NULL))
    {
        add_line("Unable to read data");
        serial_ok = false;        
        return(0);
    }
           
    // Check length of packet
    if (data_read < len)
    {
//        if (!timeout_flag)
        {
            add_line("Read data timeout in serial");            
            timeout_flag = true;
        }
       
        serial_ok = false;        
//        return(-1);
    }
    
    show_buffer("READ: ", buffer, data_read, 1);

    return (data_read);
}






/* Starts the serial system */
void start_serial( unsigned int baud)
{
//	serial_ok = false;
  
	h_serial = CreateFile(serial_port,							// Port name 
			 			  GENERIC_READ | GENERIC_WRITE,		// File access
		                  0,								// share mode
			 			  NULL,								// Security
			 			  OPEN_ALWAYS,						// Creation					
			 			  0,								// Flags
						  NULL );							// Template

	if (h_serial == INVALID_HANDLE_VALUE)
	{
        char b[400];
        sprintf(b, "Unable to open serial port (%s)", serial_port);
        add_line(b);

		return;
	}

	DCB serial_param;

	if (!GetCommState(h_serial, &serial_param))
	{
		add_line("Unable to get commstate");
		return;
	}

    serial_param.BaudRate = baud;
	serial_param.fParity = false;	
    serial_param.fBinary  = true;
	serial_param.fOutxCtsFlow = false;
	serial_param.fOutxDsrFlow = false;
	serial_param.fDtrControl = DTR_CONTROL_DISABLE;
    serial_param.fDsrSensitivity = false;
    serial_param.fErrorChar = false;
    serial_param.fNull = false;
	serial_param.fOutX = false;
	serial_param.fInX = false;
    serial_param.fRtsControl = RTS_CONTROL_DISABLE;		// Disable request to send control
    serial_param.Parity = NOPARITY;
    serial_param.ByteSize = 8;
	serial_param.StopBits = ONESTOPBIT;

	if (!SetCommState(h_serial, &serial_param))
	{
		add_line("Unable to set commstate");
		return;
	}

	COMMTIMEOUTS timeouts;

	timeouts.ReadIntervalTimeout = 500;
	
	timeouts.ReadTotalTimeoutMultiplier = 1;
	timeouts.ReadTotalTimeoutConstant = 500; 
	timeouts.WriteTotalTimeoutMultiplier = 1;
	timeouts.WriteTotalTimeoutConstant = 500;

	if (!SetCommTimeouts(h_serial, &timeouts))
	{
		add_line("Unable to get comm timeouts");
		return;
	}
	
	flush_serial();
	
	add_line("Serial Online");
}
/* End of start serial */



/* Close the serial port */
void stop_serial( void )
{    
   

    if (h_serial != NULL)
    	CloseHandle(h_serial);
}
/* End of stop serial */


void serial_reset( void )
{



}















